import os, sys, json, subprocess, re

def osa_prompt(message, title='', buttons=['OK']):
  process = subprocess.Popen(
    [
      'osascript',
      '-e',
      'display dialog "%s" buttons {%s} with title "%s"' % (
        json.dumps(message)[1:-1],
        json.dumps(buttons)[1:-1],
        json.dumps(title)[1:-1]
      )
    ],
    stdout=subprocess.PIPE,
    stderr=subprocess.PIPE
  )
  output, error = process.communicate()
  return output[output.index(':')+1:].strip()

def osa_input(message, title='', defaultValue=''):
  process = subprocess.Popen(
    [
      'osascript',
      '-e',
      'display dialog "%s" default answer "%s" with title "%s"' % (
        json.dumps(message)[1:-1],
        json.dumps(defaultValue)[1:-1],
        json.dumps(title)[1:-1]
      ),
      '-e',
      'text returned of result'
    ],
    stdout=subprocess.PIPE,
    stderr=subprocess.PIPE
  )
  output, error = process.communicate()
  return output.strip()

def osa_choose_file(prompt='Select a file:', file_types=[]):
  process = subprocess.Popen(
    [
      'osascript',
      '-e',
      'return POSIX path of (choose file with prompt "%s" of type {%s})' % (
        json.dumps(prompt)[1:-1],
        json.dumps(file_types)[1:-1]
      )
    ],
    stdout=subprocess.PIPE,
    stderr=subprocess.PIPE
  )
  output, error = process.communicate()
  return output.strip()

def osa_choose_from_list(list_items, prompt='Select a list item:', default_index=None):
  process = subprocess.Popen(
    [
      'osascript',
      '-e',
      'choose from list {%s} with prompt "%s"%s' % (
        json.dumps(list_items)[1:-1],
        json.dumps(prompt)[1:-1],
        '' if default_index is None else (' default items {%s}' % json.dumps(list_items[default_index]))
      )
    ],
    stdout=subprocess.PIPE,
    stderr=subprocess.PIPE
  )
  output, error = process.communicate()
  output = output.strip()
  return None if output == 'false' else output

def save_settings(settings):
  with open(get_settings_path(), 'w') as file:
    file.write(json.dumps(settings))

def read_settings():
  json_path = get_settings_path()
  if not os.path.exists(json_path):
    save_settings({"libs":[],"set_wine_prefix":True,"wine_path":None})
  with open(json_path, 'r') as file:
    return json.loads(file.read())

def get_settings_path():
  return os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), 'wtlibraries.json')

def open_wtlib(wtlib, settings):
  os.putenv('WINEPREFIX', re.sub(r'(?i)^(.+)/drive_[a-z]/.+$', '\\1', wtlib['path']))
  subprocess.Popen(
    [settings['wine_path'] or '/Applications/Wine.app/Contents/Resources/bin/wine', wtlib['path']],
    stdout=subprocess.PIPE,
    stderr=subprocess.PIPE
  )

def main():
  settings = read_settings()
  wtlibs = settings['libs']

  # Continue looping until a new library is added and opened, or a library is opened, or Cancel is picked
  while True:
    opt_other = 'Other Options'

    # Ask which version should be opened if any exist
    response = opt_other
    if len(wtlibs) > 0:
      response = osa_choose_from_list(
        [ 'Open %s' % (lib['name']) for lib in wtlibs ] + [opt_other],
        'What would you like to do?',
        default_index=settings['last_lib_index']
      )

    # Advanced option (automatically executed if no versions exist)
    if response == opt_other:
      opt_add = 'Add Library'
      opt_remove = 'Remove Library' if len(wtlibs) else False
      opt_wine_prefix = '%s WINE_PREFIX' % ('Unset' if settings['set_wine_prefix'] else 'Set')
      opt_wine_path = 'Set Wine Path'
      options = [v for v in [opt_add, opt_remove, opt_wine_prefix, opt_wine_path] if v]
      response = osa_choose_from_list(options, 'What would you like to do?')

      # Add a new library
      if response == opt_add:
        wtlib_path = osa_choose_file('Select the desired WTLibrary.exe file:', ['exe'])
        if wtlib_path:
          wtlib_name = osa_input(
            '%s\n\nWhat would you like this version of Watchtower Library to be called?  (eg. Spanish)' % wtlib_path,
            'Version %s' % (len(wtlibs) + 1)
          )
          if len(wtlib_name) > 0:
            wtlib = { 'name': wtlib_name, 'path': wtlib_path }
            settings['last_lib_index'] = len(wtlibs)
            wtlibs.append(wtlib)
            save_settings(settings)
            return open_wtlib(wtlib, settings)

      # Remove a library
      elif response == opt_remove:
        response = osa_choose_from_list([lib['name'] for lib in wtlibs], 'Which version of the Watchtower Library do you want to remove?')
        if response:
          new_wtlibs = []
          for index, wtlib in enumerate(wtlibs):
            if wtlib['name'] == response:
              removed_index = index
            else:
              new_wtlibs.append(wtlib)
          if removed_index == settings['last_lib_index']:
            settings['last_lib_index'] = None
          elif removed_index < settings['last_lib_index']:
            settings['last_lib_index'] -= 1
          wtlibs = new_wtlibs
          settings['libs'] = wtlibs
          save_settings(settings)

      # Set/unset the wine prefix
      elif response == opt_wine_prefix:
        settings['set_wine_prefix'] = not settings['set_wine_prefix']
        save_settings(settings)

      # Update wine_path
      elif response == opt_wine_path:
        settings['wine_path'] = osa_input('Enter the path to the wine:', settings['wine_path'] or '')
        save_settings(settings)

      # Cancel
      else:
        return

    # Open desired version
    elif response:
      response = response.replace('Open ', '', 1)
      for index, wtlib in enumerate(wtlibs):
        if wtlib['name'] == response:
          settings['last_lib_index'] = index
          save_settings(settings)
          return open_wtlib(wtlib, settings)

    # Cancel
    else:
      return

main()
